<?php
// file_backup.php

// Increase script limits and enable error reporting
set_time_limit(0);
ini_set('memory_limit', '1024M');
ini_set('display_errors', 1);
error_reporting(E_ALL);

$currentDir = realpath('.');
$debug_log = [];

// Function to push condensed debug messages to the log
function pushLog($msg) {
    global $debug_log;
    $debug_log[] = date('[Y-m-d H:i:s] ') . $msg;
}

pushLog("File backup script started.");

// Condensed recursive function to add a folder's contents to the ZIP archive
function addFolderToZip($folder, &$zip, $zipPath, $excludeArchives = false, $excludeLarge = false, $customExclusions = array()) {
    $fileCount = 0;
    $excludedCount = 0;
    $iterator = new RecursiveIteratorIterator(
        new RecursiveDirectoryIterator($folder, FilesystemIterator::SKIP_DOTS),
        RecursiveIteratorIterator::SELF_FIRST
    );
    foreach ($iterator as $file) {
        $localPath = $zipPath . DIRECTORY_SEPARATOR . $iterator->getSubPathName();
        if ($file->isDir()) {
            $zip->addEmptyDir($localPath);
        } else {
            $extension = strtolower(pathinfo($file->getFilename(), PATHINFO_EXTENSION));
            $skip = false;
            if ($excludeArchives && in_array($extension, ['zip','tar'])) { $skip = true; }
            if ($excludeLarge && $file->getSize() > 10485760) { $skip = true; }
            if (!empty($customExclusions) && in_array($extension, $customExclusions)) { $skip = true; }
            if ($skip) {
                $excludedCount++;
                continue;
            }
            $zip->addFile($file->getRealPath(), $localPath);
            $fileCount++;
        }
    }
    pushLog("Folder '$folder' processed: $fileCount files added, $excludedCount files excluded.");
}

// If ?list=1 is passed via GET, return the updated ZIP file list as HTML.
if (isset($_GET['list']) && $_GET['list'] == 1) {
    $zipFiles = glob('*.zip');
    usort($zipFiles, function($a, $b) { return filectime($b) - filectime($a); });
    ob_start();
    ?>
    <table class="striped responsive-table">
      <thead>
          <tr>
              <th>#</th>
              <th>File Name</th>
              <th>Created Date</th>
              <th class="right-align">Size (MB)</th>
              <th class="center-align">Actions</th>
          </tr>
      </thead>
      <tbody>
          <?php 
          $i = 1;
          foreach ($zipFiles as $file): ?>
          <tr>
              <td><?= $i++ ?></td>
              <td><a href="<?= htmlspecialchars($file) ?>" download><?= htmlspecialchars($file) ?></a></td>
              <td><?= date("Y-m-d H:i:s", filectime($file)) ?></td>
              <td class="right-align"><?= round(filesize($file) / 1048576, 2) ?></td>
              <td class="center-align">
                  <a href="<?= htmlspecialchars($file) ?>" class="btn-small" download>
                    <i class="material-icons" style="font-size:18px;">cloud_download</i>
                  </a>
                  <button class="btn-small red deleteBtn" data-file="<?= htmlspecialchars($file) ?>">
                    <i class="material-icons" style="font-size:18px;">delete</i>
                  </button>
                  <button class="btn-small copyUrlBtn" data-url="<?= htmlspecialchars($file) ?>">
                    <i class="material-icons" style="font-size:18px;">content_copy</i>
                  </button>
              </td>
          </tr>
          <?php endforeach; ?>
      </tbody>
    </table>
    <?php
    $html = ob_get_clean();
    echo $html;
    exit;
}

// Handle DELETE action for ZIP files
if ($_SERVER['REQUEST_METHOD'] === 'POST' && isset($_POST['delete_zip'])) {
    $fileToDelete = basename($_POST['delete_zip']);
    $filePath = $currentDir . DIRECTORY_SEPARATOR . $fileToDelete;
    $response = [];
    pushLog("Attempting to delete ZIP file: " . $fileToDelete);
    if (file_exists($filePath) && pathinfo($filePath, PATHINFO_EXTENSION) === 'zip') {
        if (unlink($filePath)) {
            $response['success'] = "Deleted ZIP file: " . htmlspecialchars($fileToDelete);
            pushLog("Successfully deleted ZIP file: " . $fileToDelete);
        } else {
            $response['error'] = "Failed to delete ZIP file: " . htmlspecialchars($fileToDelete);
            pushLog("Error deleting ZIP file: " . $fileToDelete);
        }
    } else {
        $response['error'] = "ZIP file not found or invalid: " . htmlspecialchars($fileToDelete);
        pushLog("ZIP file not found or invalid: " . $fileToDelete);
    }
    $response['debug'] = implode("\n", $debug_log);
    header('Content-Type: application/json');
    echo json_encode($response);
    exit;
}

// Handle ZIP creation (File Backup Only)
if ($_SERVER['REQUEST_METHOD'] === 'POST' && isset($_POST['folders'])) {
    $selectedFolders = $_POST['folders'];
    $validFiles = [];   // Files in root
    $validFolders = []; // Subfolders to include

    // Build custom exclusions array from form inputs
    $customExclusions = array();
    if (isset($_POST['exclude_images'])) {
        $customExclusions = array_merge($customExclusions, ['jpg','jpeg','png','gif','svg']);
    }
    if (isset($_POST['exclude_videos'])) {
        $customExclusions = array_merge($customExclusions, ['mp4','avi','mov','mkv','wmv']);
    }
    if (isset($_POST['exclude_custom']) && !empty(trim($_POST['exclude_custom']))) {
        $customList = explode(',', $_POST['exclude_custom']);
        foreach ($customList as $ext) {
            $ext = trim(strtolower($ext));
            if (!empty($ext)) {
                $customExclusions[] = $ext;
            }
        }
    }
    pushLog("Custom exclusions: " . implode(", ", $customExclusions));

    // Check if __ROOT__ is selected
    if (in_array('__ROOT__', $selectedFolders)) {
        pushLog("Root folder option selected. Including all files in root.");
        foreach (scandir($currentDir) as $item) {
            $itemPath = $currentDir . DIRECTORY_SEPARATOR . $item;
            if (is_file($itemPath)) {
                $ext = strtolower(pathinfo($item, PATHINFO_EXTENSION));
                if (isset($_POST['exclude_archives']) && in_array($ext, ['zip','tar'])) {
                    pushLog("Excluded root file (archive): " . $item);
                    continue;
                }
                if (isset($_POST['exclude_large']) && filesize($itemPath) > 10485760) {
                    pushLog("Excluded root file (large): " . $item);
                    continue;
                }
                if (!empty($customExclusions) && in_array($ext, $customExclusions)) {
                    pushLog("Excluded root file (custom): " . $item);
                    continue;
                }
                $validFiles[] = ['path' => $itemPath, 'prefix' => ''];
                pushLog("Included root file: " . $item);
            }
        }
        $allSubfolders = array_filter(scandir($currentDir), function($item) use ($currentDir) {
            return is_dir($currentDir . DIRECTORY_SEPARATOR . $item) && !in_array($item, ['.', '..']);
        });
        // Auto-select all subfolders if root is selected; user may manually uncheck.
        foreach ($allSubfolders as $folder) {
            if (in_array($folder, $selectedFolders)) {
                $folderPath = realpath($folder);
                if ($folderPath) {
                    $validFolders[] = ['path' => $folderPath, 'prefix' => $folder];
                    pushLog("Included subfolder: " . $folder);
                }
            } else {
                pushLog("Manually excluded subfolder: " . $folder);
            }
        }
    } else {
        // If __ROOT__ is not selected, include only the explicitly selected folders.
        foreach ($selectedFolders as $folder) {
            $target = realpath($folder);
            if ($target && is_dir($target) && strpos($target, $currentDir) === 0) {
                $validFolders[] = ['path' => $target, 'prefix' => basename($target)];
                pushLog("Added folder: " . $target);
            } else {
                pushLog("Folder '$folder' is invalid or outside the current directory.");
            }
        }
    }

    //CHANGE HERE
    $siteName = preg_replace('/[^a-zA-Z0-9]/', '', basename($currentDir)); // Sanitize directory name
    if (empty($siteName)) {
        $siteName = 'root'; // Default if directory name is invalid/empty
    }
    $zipName = $siteName . '_filebackup_' . date('Ymd_His') . '.zip';
    $zipPath = $currentDir . DIRECTORY_SEPARATOR . $zipName;
    pushLog("Creating ZIP file: " . $zipPath);
    
    $zip = new ZipArchive();
    if ($zip->open($zipPath, ZipArchive::CREATE | ZipArchive::OVERWRITE) === TRUE) {
        foreach ($validFiles as $fileInfo) {
            $filePath = $fileInfo['path'];
            $localName = basename($filePath);
            $zip->addFile($filePath, $localName);
        }
        foreach ($validFolders as $entry) {
            pushLog("Adding folder to ZIP: " . $entry['path'] . " as " . $entry['prefix']);
            addFolderToZip($entry['path'], $zip, $entry['prefix'], isset($_POST['exclude_archives']), isset($_POST['exclude_large']), $customExclusions);
        }
        pushLog("Closing ZIP file.");
        if ($zip->close()) {
            pushLog("ZIP file closed successfully.");
            $response['success'] = "ZIP created successfully: " . htmlspecialchars($zipName);
        } else {
            pushLog("Error closing ZIP file. Status: " . $zip->status . " System status: " . $zip->statusSys);
            $response['error'] = "Error closing ZIP file";
        }
    } else {
        pushLog("Failed to open ZIP file: " . $zipPath);
        $response['error'] = "Failed to create ZIP file";
    }
    
    $response['debug'] = implode("\n", $debug_log);
    header('Content-Type: application/json');
    echo json_encode($response);
    exit;
}

// Get available ZIP files and subfolders for display
$zipFiles = glob('*.zip');
usort($zipFiles, function($a, $b) { return filectime($b) - filectime($a); });
$subFolders = array_filter(scandir('.'), function($item) use ($currentDir) {
    return is_dir($currentDir . DIRECTORY_SEPARATOR . $item) && !in_array($item, ['.', '..']);
});
?>
<!DOCTYPE html>
<html>
<head>
    <meta charset="UTF-8">
    <title>File Backup - Developed by Murad Khan</title>
    <link href="https://fonts.googleapis.com/icon?family=Material+Icons" rel="stylesheet">
    <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/font-awesome/6.4.0/css/all.min.css">
    <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/materialize/1.0.0/css/materialize.min.css">
    <link rel="icon" href="https://via.placeholder.com/32" type="image/png">
    <style>
        body { padding: 20px; }
        .container { margin-top: 30px; }
        ul.folder-tree { list-style-type: none; padding-left: 20px; }
        ul.folder-tree li { margin: 4px 0; }
        ul.folder-tree li ul { padding-left: 20px; }
        /* Custom Checkbox Styling */
        input[type="checkbox"] {
            -webkit-appearance: none;
            -moz-appearance: none;
            appearance: none;
            border: 2px solid #ccc;
            width: 20px;
            height: 20px;
            border-radius: 3px;
            cursor: pointer;
            margin-right: 10px;
            position: relative;
            vertical-align: middle;
        }
        input[type="checkbox"]:checked {
            background-color: #4caf50;
            border: 2px solid #4caf50;
        }
        input[type="checkbox"]:checked:after {
            content: '\2713';
            position: absolute;
            top: 0;
            left: 4px;
            font-size: 16px;
            color: #fff;
        }
        /* Progress Overlay */
        #progressOverlay {
            display: none;
            position: fixed;
            top: 0;
            left: 0;
            width: 100%;
            height: 100%;
            background: rgba(0,0,0,0.5);
            z-index: 1000;
            text-align: center;
            padding-top: 200px;
            color: #fff;
            font-size: 20px;
        }
        #progressBarContainer {
            width: 80%;
            background: #ccc;
            margin: 20px auto;
            border-radius: 5px;
            overflow: hidden;
        }
        #progressBar {
            width: 0%;
            height: 30px;
            background: #4caf50;
            text-align: center;
            line-height: 30px;
            color: white;
        }
        table.striped thead { background-color: #4caf50; color: #fff; }
        #debugLogSection {
            margin-top: 30px;
            background: #f9f9f9;
            padding: 15px;
            border: 1px solid #ccc;
            font-family: monospace;
            font-size: 0.9em;
            max-height: 300px;
            overflow-y: auto;
        }
        /* Header styling */
        nav .brand-logo {
            display: flex;
            align-items: center;
        }
        nav .brand-logo i.material-icons {
            font-size: 28px;
            margin-right: 8px;
        }
        nav ul li {
            margin: 0 10px;
        }
        nav ul li a, nav ul li span {
            font-size: 14px;
            display: flex;
            align-items: center;
        }
        nav ul li a i {
            font-size: 16px;
            margin-right: 4px;
        }
    </style>
</head>
<body>
    <!-- Header Navigation -->
    <nav class="blue">
      <div class="nav-wrapper container">
        <a href="#" class="brand-logo">
          <i class="material-icons">backup</i> File Backup System
        </a>
        <ul id="nav-mobile" class="right hide-on-med-and-down">
          <li>
            <a href="https://techoneer.pk" target="_blank">
              <i class="fas fa-globe"></i> Website
            </a>
          </li>
          <li>
            <a href="https://wa.me/923444595233" target="_blank">
              <i class="fab fa-whatsapp"></i> WhatsApp
            </a>
          </li>
          <li>
            <a href="https://facebook.com/muradgull" target="_blank">
              <i class="fab fa-facebook-f"></i> Facebook
            </a>
          </li>
          <li><span>Developed by Murad Khan</span></li>
          <li><span>Version 1.0</span></li>
		  
		  <li>
            <a href="zipdb.php" target="_blank">
              <i class="fab fa-facebook-f"></i> database
            </a>
          </li>
        </ul>
      </div>
    </nav>

    <div class="container">
        <h4>File Backup (ZIP Creation)</h4>
        <p>
            <strong>Note:</strong> Check "Root Folder" to back up the entire site (all root files will be included) and auto-select all subfolders.
            You may then manually uncheck any folder you do not want.
        </p>
        <form id="zipForm" method="post">
            <div class="row">
                <div class="col s6">
                    <h6>Include Folders</h6>
                    <ul class="folder-tree">
                        <li>
                            <label>
                                <input type="checkbox" id="rootCheckbox" name="folders[]" value="__ROOT__" />
                                <span>Root Folder (<?= htmlspecialchars(basename($currentDir) ? basename($currentDir) : "root") ?>)</span>
                            </label>
                        </li>
                        <li>
                            <strong>Or select individual folders:</strong>
                            <ul>
                                <?php foreach ($subFolders as $folder): ?>
                                <li>
                                    <label>
                                        <input type="checkbox" class="folderCheckbox" name="folders[]" value="<?= htmlspecialchars($folder) ?>" />
                                        <span><?= htmlspecialchars($folder) ?></span>
                                    </label>
                                </li>
                                <?php endforeach; ?>
                            </ul>
                        </li>
                    </ul>
                </div>
                <div class="col s6">
                    <h6>Exclusion Options</h6>
                    <p>
                        <label>
                            <input type="checkbox" name="exclude_archives" value="1" />
                            <span>Exclude ZIP/TAR files</span>
                        </label>
                    </p>
                    <p>
                        <label>
                            <input type="checkbox" name="exclude_large" value="1" />
                            <span>Exclude large files (over 10 MB)</span>
                        </label>
                    </p>
                    <p>
                        <label>
                            <input type="checkbox" name="exclude_images" value="1" />
                            <span>Exclude Images</span>
                        </label>
                    </p>
                    <p>
                        <label>
                            <input type="checkbox" name="exclude_videos" value="1" />
                            <span>Exclude Videos</span>
                        </label>
                    </p>
                    <p>
                        <label>Additional Exclusions (comma-separated extensions):</label>
                        <input type="text" name="exclude_custom" placeholder="e.g., pdf,docx" />
                    </p>
                </div>
            </div>
            <button class="btn waves-effect waves-light" type="submit">
                <i class="material-icons left">archive</i> Create ZIP File
            </button>
        </form>
        <div id="progressOverlay">
            <div>Creating ZIP... Please wait.</div>
            <div id="progressBarContainer">
                <div id="progressBar">0%</div>
            </div>
        </div>
        <div id="zipListSection">
            <h5>Available ZIP Files:</h5>
            <div id="zipListContent">
                <?php if (!empty($zipFiles)): ?>
                <table class="striped responsive-table">
                    <thead>
                        <tr>
                            <th>#</th>
                            <th>File Name</th>
                            <th>Created Date</th>
                            <th class="right-align">Size (MB)</th>
                            <th class="center-align">Actions</th>
                        </tr>
                    </thead>
                    <tbody>
                        <?php $i = 1; foreach ($zipFiles as $file): ?>
                        <tr>
                            <td><?= $i++ ?></td>
                            <td>
                              <a href="<?= htmlspecialchars($file) ?>" download><?= htmlspecialchars($file) ?></a>
                            </td>
                            <td><?= date("Y-m-d H:i:s", filectime($file)) ?></td>
                            <td class="right-align"><?= round(filesize($file) / 1048576, 2) ?></td>
                            <td class="center-align">
                                <a href="<?= htmlspecialchars($file) ?>" class="btn-small" download>
                                  <i class="material-icons">cloud_download</i>
                                </a>
                                <button class="btn-small red deleteBtn" data-file="<?= htmlspecialchars($file) ?>">
                                  <i class="material-icons">delete</i>
                                </button>
                                <button class="btn-small copyUrlBtn" data-url="<?= htmlspecialchars($file) ?>">
                                  <i class="material-icons">content_copy</i>
                                </button>
                            </td>
                        </tr>
                        <?php endforeach; ?>
                    </tbody>
                </table>
                <?php else: ?>
                <p>No ZIP files available.</p>
                <?php endif; ?>
            </div>
        </div>
        <div id="debugLogSection">
            <h6>Debug Log:</h6>
            <pre id="debugLogContent"><?= !empty($debug_log) ? implode("\n", $debug_log) : "No debug log messages." ?></pre>
        </div>
    </div>
    <script src="https://cdnjs.cloudflare.com/ajax/libs/materialize/1.0.0/js/materialize.min.js"></script>
    <script>
        // On page load, if Root Folder is checked, auto-check subfolder checkboxes.
        document.addEventListener("DOMContentLoaded", function() {
            var rootChk = document.getElementById('rootCheckbox');
            if(rootChk && rootChk.checked) {
                document.querySelectorAll('.folderCheckbox').forEach(function(chk) {
                    chk.checked = true;
                });
            }
        });
        // When "Root Folder" is checked, auto-check all subfolder checkboxes.
        document.getElementById('rootCheckbox').addEventListener('change', function() {
            var checked = this.checked;
            document.querySelectorAll('.folderCheckbox').forEach(function(chk) {
                chk.checked = checked;
            });
        });
        // Function to refresh the ZIP file list via AJAX (with cache busting)
        function refreshZipList() {
            fetch(window.location.href + "?list=1&rand=" + Math.random(), {
                method: 'GET',
                headers: { 'X-Requested-With': 'XMLHttpRequest' }
            })
            .then(response => response.text())
            .then(html => {
                document.getElementById('zipListContent').innerHTML = html;
            })
            .catch(error => {
                M.toast({html: 'Error refreshing ZIP list: ' + error, classes: 'red'});
            });
        }
        // Handle ZIP creation via AJAX
        document.getElementById('zipForm').addEventListener('submit', function(e) {
            e.preventDefault();
            var overlay = document.getElementById('progressOverlay');
            var progressBar = document.getElementById('progressBar');
            overlay.style.display = 'block';
            var progress = 0;
            var interval = setInterval(function() {
                if (progress < 90) {
                    progress++;
                    progressBar.style.width = progress + '%';
                    progressBar.textContent = progress + '%';
                }
            }, 100);
            var formData = new FormData(this);
            fetch(window.location.href, {
                method: 'POST',
                body: formData,
                headers: { 'X-Requested-With': 'XMLHttpRequest' }
            })
            .then(response => response.json())
            .then(data => {
                clearInterval(interval);
                progressBar.style.width = '100%';
                progressBar.textContent = '100%';
                setTimeout(() => { overlay.style.display = 'none'; }, 500);
                M.toast({html: data.error ? "Error: " + data.error : data.success, classes: data.error ? 'red' : 'green'});
                // Update Debug Log section with returned log
                if(data.debug){
                    document.getElementById('debugLogContent').textContent = data.debug;
                }
                refreshZipList();
            })
            .catch(error => {
                clearInterval(interval);
                overlay.style.display = 'none';
                M.toast({html: 'An error occurred: ' + error, classes: 'red'});
            });
        });
        // Use event delegation for delete buttons in the ZIP list
        document.getElementById('zipListSection').addEventListener('click', function(e) {
            if (e.target && e.target.closest('.deleteBtn')) {
                var btn = e.target.closest('.deleteBtn');
                var fileName = btn.getAttribute('data-file');
                if (confirm("Are you sure you want to delete " + fileName + "?")) {
                    var formData = new FormData();
                    formData.append('delete_zip', fileName);
                    fetch(window.location.href, {
                        method: 'POST',
                        body: formData,
                        headers: { 'X-Requested-With': 'XMLHttpRequest' }
                    })
                    .then(response => response.json())
                    .then(data => {
                        M.toast({html: data.error ? "Error: " + data.error : data.success, classes: data.error ? 'red' : 'green'});
                        if (!data.error) { refreshZipList(); }
                    })
                    .catch(error => {
                        M.toast({html: 'An error occurred: ' + error, classes: 'red'});
                    });
                }
            }
        });
        // Use event delegation for copy URL buttons in the ZIP list
        document.getElementById('zipListSection').addEventListener('click', function(e) {
            if (e.target && e.target.closest('.copyUrlBtn')) {
                var btn = e.target.closest('.copyUrlBtn');
                var url = btn.getAttribute('data-url');
                // Construct the full URL using the base directory (excluding the PHP file name)
                var baseUrl = window.location.href.split('/').slice(0, -1).join('/');
                var fullUrl = baseUrl + "/" + url;
                navigator.clipboard.writeText(fullUrl)
                .then(() => {
                    M.toast({html: 'Download URL copied to clipboard!', classes: 'green'});
                })
                .catch(err => {
                    M.toast({html: 'Failed to copy URL: ' + err, classes: 'red'});
                });
            }
        });
    </script>
</body>
</html>