<?php
ob_start(); // Start output buffering

// db_backup.php

// Increase script limits and enable error reporting
set_time_limit(0);
ini_set('memory_limit', '1024M');
// For production, consider setting display_errors to 0
ini_set('display_errors', 1);
error_reporting(E_ALL);

$currentDir = realpath('.');
$debug_log = [];

// Helper function to push debug messages
function pushLog($msg) {
    global $debug_log;
    $debug_log[] = date('[Y-m-d H:i:s] ') . $msg;
}
pushLog("Multi-CMS database backup script started.");

// --- Helper: Get Site Name ---
function getSiteName() {
    // Use HTTP_HOST as site name and replace dots with underscores
    $host = $_SERVER['HTTP_HOST'] ?? 'site';
    return preg_replace('/\./', '_', $host);
}

// --- Supported CMS Configuration ---
$cmsList = array(
    "wordpress"  => array("config" => "wp-config.php", "label" => "WordPress"),
    "joomla"     => array("config" => "configuration.php", "label" => "Joomla"),
    "prestashop" => array("config" => "config/settings.inc.php", "label" => "PrestaShop"),
    "opencart"   => array("config" => "config.php", "label" => "OpenCart")
);

// --- CMS Test & Backup Functions ---

// Test connection for a given CMS (WordPress and Joomla implemented)
function testConnectionCMS($cms) {
    global $cmsList;
    $result = "";
    switch ($cms) {
        case "wordpress":
            if (!file_exists($cmsList[$cms]['config'])) {
                $result = "wp-config.php not found.";
            } else {
                $config = file_get_contents($cmsList[$cms]['config']);
                $db_name = $db_user = $db_password = "";
                $db_host = "localhost";
                if (preg_match("/define\(\s*[\"']DB_NAME[\"']\s*,\s*[\"'](.+?)[\"']\s*\)/i", $config, $matches))
                    $db_name = $matches[1];
                if (preg_match("/define\(\s*[\"']DB_USER[\"']\s*,\s*[\"'](.+?)[\"']\s*\)/i", $config, $matches))
                    $db_user = $matches[1];
                if (preg_match("/define\(\s*[\"']DB_PASSWORD[\"']\s*,\s*[\"'](.*?)?[\"']\s*\)/i", $config, $matches))
                    $db_password = $matches[1];
                if (preg_match("/define\(\s*[\"']DB_HOST[\"']\s*,\s*[\"'](.+?)[\"']\s*\)/i", $config, $matches))
                    $db_host = $matches[1];
                if (empty($db_name) || empty($db_user)) {
                    $result = "Required DB credentials missing in wp-config.php.";
                } else {
                    $conn = @mysqli_connect($db_host, $db_user, $db_password, $db_name);
                    if ($conn) {
                        mysqli_close($conn);
                        $result = "Successfully connected to WordPress database.";
                    } else {
                        $result = "Connection failed: " . mysqli_connect_error();
                    }
                }
            }
            break;
        case "joomla":
            $creds = getJoomlaDbCredentials();
            if (!$creds) {
                $result = "configuration.php not found.";
            } elseif (empty($creds['DB_NAME']) || empty($creds['DB_USER'])) {
                $result = "Required Joomla DB credentials missing.";
            } else {
                $conn = @mysqli_connect($creds['DB_HOST'], $creds['DB_USER'], $creds['DB_PASSWORD'], $creds['DB_NAME']);
                if ($conn) {
                    mysqli_close($conn);
                    $result = "Successfully connected to Joomla database.";
                } else {
                    $result = "Connection failed: " . mysqli_connect_error();
                }
            }
            break;
        case "prestashop":
            $result = "PrestaShop backup not implemented.";
            break;
        case "opencart":
            $result = "OpenCart backup not implemented.";
            break;
        default:
            $result = "Unknown CMS.";
    }
    pushLog("Test for $cms: " . $result);
    return $result;
}

// --- WordPress Credential Extraction ---
function getWpDbCredentials() {
    $config_file = "wp-config.php";
    if (!file_exists($config_file)) {
        pushLog("wp-config.php not found.");
        return false;
    }
    $config = file_get_contents($config_file);
    $credentials = array();
    if (preg_match("/define\(\s*[\"']DB_NAME[\"']\s*,\s*[\"'](.+?)[\"']\s*\)/i", $config, $matches))
        $credentials['DB_NAME'] = $matches[1];
    else
        pushLog("DB_NAME not found.");
    if (preg_match("/define\(\s*[\"']DB_USER[\"']\s*,\s*[\"'](.+?)[\"']\s*\)/i", $config, $matches))
        $credentials['DB_USER'] = $matches[1];
    else
        pushLog("DB_USER not found.");
    if (preg_match("/define\(\s*[\"']DB_PASSWORD[\"']\s*,\s*[\"'](.*?)?[\"']\s*\)/i", $config, $matches))
        $credentials['DB_PASSWORD'] = $matches[1];
    else
        pushLog("DB_PASSWORD not found.");
    if (preg_match("/define\(\s*[\"']DB_HOST[\"']\s*,\s*[\"'](.+?)[\"']\s*\)/i", $config, $matches))
        $credentials['DB_HOST'] = $matches[1];
    else {
        pushLog("DB_HOST not found; defaulting to localhost.");
        $credentials['DB_HOST'] = "localhost";
    }
    return $credentials;
}

// --- Joomla Credential Extraction ---
// Updated regex patterns (case-insensitive, allow extra whitespace)
function getJoomlaDbCredentials() {
    $config_file = "configuration.php";
    if (!file_exists($config_file)) {
        pushLog("configuration.php not found.");
        return false;
    }
    $contents = file_get_contents($config_file);
    $credentials = array();
    if (preg_match("/public\s+\$host\s*=\s*'([^']+)'/i", $contents, $matches))
        $credentials['DB_HOST'] = $matches[1];
    else {
        pushLog("Joomla DB host not found; defaulting to localhost.");
        $credentials['DB_HOST'] = "localhost";
    }
    if (preg_match("/public\s+\$user\s*=\s*'([^']+)'/i", $contents, $matches))
        $credentials['DB_USER'] = $matches[1];
    else
        pushLog("Joomla DB user not found.");
    if (preg_match("/public\s+\$password\s*=\s*'([^']*)'/i", $contents, $matches))
        $credentials['DB_PASSWORD'] = $matches[1];
    else
        pushLog("Joomla DB password not found.");
    if (preg_match("/public\s+\$db\s*=\s*'([^']+)'/i", $contents, $matches))
        $credentials['DB_NAME'] = $matches[1];
    else
        pushLog("Joomla DB name not found.");
    return $credentials;
}

// --- Backup Functions ---

// Helper: Determine mysqldump path based on OS
function getMysqldumpPath() {
    if (stripos(PHP_OS, 'WIN') === 0) {
        return 'D:\wamp64\bin\mysql\mysql5.7.44\bin\mysqldump.exe';
    } else {
        return 'mysqldump';
    }
}

// Backup for a given CMS
function backupDatabaseCMS($cms) {
    global $cmsList;
    $backup_file = "";
    $mysqldumpPath = getMysqldumpPath();
    switch ($cms) {
        case "wordpress":
            $creds = getWpDbCredentials();
            if (!$creds) {
                pushLog("Failed to retrieve WordPress credentials.");
                return false;
            }
            if (empty($creds['DB_NAME']) || empty($creds['DB_USER'])) {
                pushLog("Missing WordPress DB name or user.");
                return false;
            }
            $backup_file = 'wp_db_backup_' . date('Ymd_His') . '.sql';
            if (empty($creds['DB_PASSWORD'])) {
                $command = '"' . $mysqldumpPath . '" --host=' . escapeshellarg($creds['DB_HOST']) .
                           " --user=" . escapeshellarg($creds['DB_USER']) . " " .
                           escapeshellarg($creds['DB_NAME']) . " > " . escapeshellarg($backup_file) . " 2>&1";
            } else {
                $command = '"' . $mysqldumpPath . '" --host=' . escapeshellarg($creds['DB_HOST']) .
                           " --user=" . escapeshellarg($creds['DB_USER']) .
                           " --password=" . escapeshellarg($creds['DB_PASSWORD']) . " " .
                           escapeshellarg($creds['DB_NAME']) . " > " . escapeshellarg($backup_file) . " 2>&1";
            }
            pushLog("Running WordPress command: $command");
            $output = shell_exec($command);
            pushLog("mysqldump output: " . $output);
            if (file_exists($backup_file) && filesize($backup_file) > 0) {
                pushLog("WordPress backup created: " . filesize($backup_file) . " bytes");
                return $backup_file;
            }
            pushLog("WordPress backup failed or file is empty.");
            return false;
            break;
        case "joomla":
            $creds = getJoomlaDbCredentials();
            if (!$creds) {
                pushLog("Failed to retrieve Joomla credentials.");
                return false;
            }
            if (empty($creds['DB_NAME']) || empty($creds['DB_USER'])) {
                pushLog("Missing Joomla DB name or user.");
                return false;
            }
            $backup_file = 'joomla_db_backup_' . date('Ymd_His') . '.sql';
            if (empty($creds['DB_PASSWORD'])) {
                $command = '"' . $mysqldumpPath . '" --host=' . escapeshellarg($creds['DB_HOST']) .
                           " --user=" . escapeshellarg($creds['DB_USER']) . " " .
                           escapeshellarg($creds['DB_NAME']) . " > " . escapeshellarg($backup_file) . " 2>&1";
            } else {
                $command = '"' . $mysqldumpPath . '" --host=' . escapeshellarg($creds['DB_HOST']) .
                           " --user=" . escapeshellarg($creds['DB_USER']) .
                           " --password=" . escapeshellarg($creds['DB_PASSWORD']) . " " .
                           escapeshellarg($creds['DB_NAME']) . " > " . escapeshellarg($backup_file) . " 2>&1";
            }
            pushLog("Running Joomla command: $command");
            $output = shell_exec($command);
            pushLog("mysqldump output: " . $output);
            if (file_exists($backup_file) && filesize($backup_file) > 0) {
                pushLog("Joomla backup created: " . filesize($backup_file) . " bytes");
                return $backup_file;
            }
            pushLog("Joomla backup failed or file is empty.");
            return false;
            break;
        case "prestashop":
            pushLog("PrestaShop backup not implemented.");
            return false;
        case "opencart":
            pushLog("OpenCart backup not implemented.");
            return false;
        default:
            pushLog("Unknown CMS type for backup.");
            return false;
    }
}

// Manual backup using provided credentials with site name in filename
function backupDatabaseManual($host, $user, $password, $dbname) {
    $mysqldumpPath = getMysqldumpPath();
    if (empty($dbname) || empty($user)) {
        pushLog("Manual backup: missing database name or user.");
        return false;
    }
    $siteName = getSiteName();
    $backup_file = $siteName . "_manual_db_backup_" . date('Ymd_His') . ".sql";
    if (empty($password)) {
        $command = '"' . $mysqldumpPath . '" --host=' . escapeshellarg($host) .
                   " --user=" . escapeshellarg($user) . " " .
                   escapeshellarg($dbname) . " > " . escapeshellarg($backup_file) . " 2>&1";
    } else {
        $command = '"' . $mysqldumpPath . '" --host=' . escapeshellarg($host) .
                   " --user=" . escapeshellarg($user) .
                   " --password=" . escapeshellarg($password) . " " .
                   escapeshellarg($dbname) . " > " . escapeshellarg($backup_file) . " 2>&1";
    }
    pushLog("Running manual backup command: $command");
    $output = shell_exec($command);
    pushLog("Manual mysqldump output: " . $output);
    if (file_exists($backup_file) && filesize($backup_file) > 0) {
        pushLog("Manual backup created: " . filesize($backup_file) . " bytes");
        return $backup_file;
    }
    pushLog("Manual backup failed or file is empty.");
    return false;
}

// --- AJAX Endpoints ---
// Return backup file list if ?list=1 is set
if (isset($_GET['list']) && $_GET['list'] == 1) {
    $backupFiles = glob('*_db_backup_*.sql');
    usort($backupFiles, function($a, $b) { return filectime($b) - filectime($a); });
    ob_clean();
    ob_start();
    ?>
    <table class="striped responsive-table">
      <thead>
          <tr>
              <th>#</th>
              <th>Site</th>
              <th>File Name</th>
              <th>Created Date</th>
              <th class="right-align">Size (KB)</th>
              <th class="center-align">Actions</th>
          </tr>
      </thead>
      <tbody>
          <?php
          $i = 1;
          foreach ($backupFiles as $file):
              $site = "Unknown";
              if (strpos($file, "wp_db_backup_") === 0) {
                  $site = "WordPress";
              } elseif (strpos($file, "joomla_db_backup_") === 0) {
                  $site = "Joomla";
              } elseif (strpos($file, "manual_db_backup_") === 0) {
                  // Extract site name from manual backup file name; assumed format: sitename_manual_db_backup_YYYYMMDD_HHMMSS.sql
                  $parts = explode("_", $file);
                  $site = $parts[0];
              }
          ?>
          <tr>
              <td><?= $i++ ?></td>
              <td><?= $site ?></td>
              <td><a href="<?= htmlspecialchars($file) ?>" download><?= htmlspecialchars($file) ?></a></td>
              <td><?= date("Y-m-d H:i:s", filectime($file)) ?></td>
              <td class="right-align"><?= round(filesize($file)/1024, 2) ?></td>
              <td class="center-align">
                  <a href="<?= htmlspecialchars($file) ?>" class="btn-small" download>
                    <i class="material-icons">cloud_download</i>
                  </a>
                  <button class="btn-small red deleteBtn" data-file="<?= htmlspecialchars($file) ?>">
                    <i class="material-icons">delete</i>
                  </button>
                  <button class="btn-small copyUrlBtn" data-url="<?= htmlspecialchars($file) ?>">
                    <i class="material-icons">content_copy</i>
                  </button>
              </td>
          </tr>
          <?php endforeach; ?>
      </tbody>
    </table>
    <?php
    $html = ob_get_clean();
    echo $html;
    exit;
}

// Handle deletion of a backup file
if ($_SERVER['REQUEST_METHOD'] === 'POST' && isset($_POST['delete_zip'])) {
    $fileToDelete = basename($_POST['delete_zip']);
    $filePath = $currentDir . DIRECTORY_SEPARATOR . $fileToDelete;
    $response = [];
    pushLog("Attempting to delete backup file: " . $fileToDelete);
    if (file_exists($filePath) && pathinfo($filePath, PATHINFO_EXTENSION) === 'sql') {
        if (unlink($filePath)) {
            $response['message'] = "Deleted backup file: " . htmlspecialchars($fileToDelete);
            pushLog("Successfully deleted backup file: " . $fileToDelete);
        } else {
            $response['message'] = "Failed to delete backup file: " . htmlspecialchars($fileToDelete);
            pushLog("Error deleting backup file: " . $fileToDelete);
        }
    } else {
        $response['message'] = "Backup file not found or invalid: " . htmlspecialchars($fileToDelete);
        pushLog("Backup file not found or invalid: " . $fileToDelete);
    }
    $response['debug'] = implode("\n", $debug_log);
    ob_clean();
    header('Content-Type: application/json');
    echo json_encode($response);
    exit;
}

// Handle CMS backup
if ($_SERVER['REQUEST_METHOD'] === 'POST' && isset($_POST['backup_cms'])) {
    $selectedCMS = $_POST['backup_cms'];
    $response = [];
    $backups = array();
    foreach ($selectedCMS as $cms) {
        $backupFile = backupDatabaseCMS($cms);
        $backups[$cms] = $backupFile ? $backupFile : false;
    }
    $msg = "";
    foreach ($backups as $cms => $file) {
        if ($file)
            $msg .= ucfirst($cms) . " backup created: " . htmlspecialchars($file) . ". ";
        else
            $msg .= ucfirst($cms) . " backup failed. ";
    }
    $response['message'] = $msg;
    $response['debug'] = implode("\n", $debug_log);
    ob_clean();
    header('Content-Type: application/json');
    echo json_encode($response);
    exit;
}

// Handle manual backup
if ($_SERVER['REQUEST_METHOD'] === 'POST' && isset($_POST['manual_backup'])) {
    $host = $_POST['db_host'];
    $user = $_POST['db_user'];
    $password = $_POST['db_password'];
    $dbname = $_POST['db_name'];
    $backupFile = backupDatabaseManual($host, $user, $password, $dbname);
    $response = [];
    if ($backupFile)
        $response['message'] = "Manual backup created: " . htmlspecialchars($backupFile);
    else
        $response['message'] = "Manual backup failed or is empty.";
    $response['debug'] = implode("\n", $debug_log);
    ob_clean();
    header('Content-Type: application/json');
    echo json_encode($response);
    exit;
}

// Handle manual test (simulate connection test) – using provided password
if (isset($_GET['manual_test']) && $_GET['manual_test'] == 1) {
    $host = $_GET['host'] ?? 'localhost';
    $user = $_GET['user'] ?? '';
    $dbname = $_GET['dbname'] ?? '';
    $pass = $_GET['password'] ?? '';
    if (empty($user) || empty($dbname)) {
        $message = "Manual test failed: DB user or name missing.";
    } else {
        $conn = @mysqli_connect($host, $user, $pass, $dbname);
        if ($conn) {
            mysqli_close($conn);
            $message = "Manual test successful.";
        } else {
            $message = "Manual test failed: " . mysqli_connect_error();
        }
    }
    $response = array("message" => $message, "debug" => implode("\n", $debug_log));
    ob_clean();
    header('Content-Type: application/json');
    echo json_encode($response);
    exit;
}

// --- Render Full HTML Page (non-AJAX) ---
$backupFiles = glob('*_db_backup_*.sql');
usort($backupFiles, function($a, $b) { return filectime($b) - filectime($a); });
?>
<!DOCTYPE html>
<html>
<head>
    <meta charset="UTF-8">
    <title>Database Backup - Developed by Murad Khan</title>
    <link href="https://fonts.googleapis.com/icon?family=Material+Icons" rel="stylesheet">
    <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/font-awesome/6.4.0/css/all.min.css">
    <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/materialize/1.0.0/css/materialize.min.css">
    <link rel="icon" href="https://via.placeholder.com/32" type="image/png">
    <style>
        body { padding: 20px; }
        .container { margin-top: 30px; }
        nav .brand-logo { display: flex; align-items: center; }
        nav .brand-logo i.material-icons { font-size: 28px; margin-right: 8px; }
        nav ul li { margin: 0 10px; }
        nav ul li a, nav ul li span { font-size: 14px; display: flex; align-items: center; }
        nav ul li a i { font-size: 16px; margin-right: 4px; }
        #debugLogSection {
            margin-top: 30px;
            background: #f9f9f9;
            padding: 15px;
            border: 1px solid #ccc;
            font-family: monospace;
            font-size: 0.9em;
            max-height: 300px;
            overflow-y: auto;
        }
        table.striped thead { background-color: #4caf50; color: #fff; }
    </style>
</head>
<body>
    <!-- Header Navigation -->
    <nav class="blue">
      <div class="nav-wrapper container">
        <a href="#" class="brand-logo">
          <i class="material-icons">storage</i> Database Backup
        </a>
        <ul id="nav-mobile" class="right hide-on-med-and-down">
          <li>
            <a href="https://techoneer.pk" target="_blank">
              <i class="fas fa-globe"></i> Website
            </a>
          </li>
          <li>
            <a href="https://wa.me/923444595233" target="_blank">
              <i class="fab fa-whatsapp"></i> WhatsApp
            </a>
          </li>
          <li>
            <a href="https://facebook.com/muradgull" target="_blank">
              <i class="fab fa-facebook-f"></i> Facebook
            </a>
          </li>
          <li><span>Developed by Murad Khan</span></li>
          <li><span>Version 1.0</span></li>
		  
		  <li>
            <a href="zipbk.php" target="_blank">
              <i class="fab fa-file"></i> File
            </a>
          </li>
        </ul>
      </div>
    </nav>

    <div class="container">
        <h4>Multi-CMS Database Backup</h4>
        <p>Automatically detect major CMS configuration files and test connection. You can also enter manual database connection details.</p>
        <!-- CMS Backup Section -->
        <div class="card">
            <div class="card-content">
                <span class="card-title">CMS Backup Options</span>
                <form id="cmsBackupForm">
                    <?php foreach ($cmsList as $key => $info): 
                        $configExists = file_exists($info['config']);
                    ?>
                    <p>
                        <label>
                            <input type="checkbox" name="backup_cms[]" value="<?= $key ?>" <?= $configExists ? "checked" : "" ?> />
                            <span><?= $info['label'] ?> <?= $configExists ? '<small class="green-text">(Config available)</small>' : '<small class="red-text">(Config missing)</small>' ?></span>
                        </label>
                        <button type="button" class="btn-small testCmsBtn" data-cms="<?= $key ?>">Test</button>
                    </p>
                    <?php endforeach; ?>
                    <button class="btn" type="submit"><i class="material-icons left">backup</i> Backup Selected CMS</button>
                </form>
            </div>
        </div>
        <br>
        <!-- Manual Backup Section -->
        <div class="card">
            <div class="card-content">
                <span class="card-title">Manual Database Backup</span>
                <form id="manualBackupForm">
                    <div class="row">
                        <div class="input-field col s12 m3">
                            <input type="text" name="db_host" id="db_host" value="localhost">
                            <label for="db_host" class="active">DB Host</label>
                        </div>
                        <div class="input-field col s12 m3">
                            <input type="text" name="db_user" id="db_user">
                            <label for="db_user">DB User</label>
                        </div>
                        <div class="input-field col s12 m3">
                            <input type="text" name="db_password" id="db_password">
                            <label for="db_password">DB Password</label>
                        </div>
                        <div class="input-field col s12 m3">
                            <input type="text" name="db_name" id="db_name">
                            <label for="db_name">DB Name</label>
                        </div>
                    </div>
                    <button class="btn" type="submit"><i class="material-icons left">backup</i> Backup Manually</button>
                    <button type="button" class="btn grey" id="testManualBtn"><i class="material-icons left">settings_input_component</i> Test Connection</button>
                </form>
                <div id="manualResponse" style="margin-top:15px;"></div>
            </div>
        </div>
        <br>
        <!-- Backup Files List -->
        <div id="backupListSection">
            <h5>Available Backup Files:</h5>
            <div id="backupListContent">
                <?php if (!empty($backupFiles)): ?>
                <table class="striped responsive-table">
                    <thead>
                        <tr>
                            <th>#</th>
                            <th>Site</th>
                            <th>File Name</th>
                            <th>Created Date</th>
                            <th class="right-align">Size (KB)</th>
                            <th class="center-align">Actions</th>
                        </tr>
                    </thead>
                    <tbody>
                        <?php
                        $i = 1;
                        foreach ($backupFiles as $file):
                            $site = "Unknown";
                            if (strpos($file, "wp_db_backup_") === 0) {
                                $site = "WordPress";
                            } elseif (strpos($file, "joomla_db_backup_") === 0) {
                                $site = "Joomla";
                            } elseif (strpos($file, "manual_db_backup_") === 0) {
                                $parts = explode("_", $file);
                                $site = $parts[0];
                            }
                        ?>
                        <tr>
                            <td><?= $i++ ?></td>
                            <td><?= $site ?></td>
                            <td><a href="<?= htmlspecialchars($file) ?>" download><?= htmlspecialchars($file) ?></a></td>
                            <td><?= date("Y-m-d H:i:s", filectime($file)) ?></td>
                            <td class="right-align"><?= round(filesize($file)/1024, 2) ?></td>
                            <td class="center-align">
                                <a href="<?= htmlspecialchars($file) ?>" class="btn-small" download>
                                    <i class="material-icons">cloud_download</i>
                                </a>
                                <button class="btn-small red deleteBtn" data-file="<?= htmlspecialchars($file) ?>">
                                    <i class="material-icons">delete</i>
                                </button>
                                <button class="btn-small copyUrlBtn" data-url="<?= htmlspecialchars($file) ?>">
                                    <i class="material-icons">content_copy</i>
                                </button>
                            </td>
                        </tr>
                        <?php endforeach; ?>
                    </tbody>
                </table>
                <?php else: ?>
                <p>No backup files available.</p>
                <?php endif; ?>
            </div>
        </div>
        <!-- Debug Log Section -->
        <div id="debugLogSection">
            <h6>Debug Log:</h6>
            <pre id="debugLogContent"><?= !empty($debug_log) ? implode("\n", $debug_log) : "No debug log messages." ?></pre>
        </div>
    </div>
    
    <script src="https://cdnjs.cloudflare.com/ajax/libs/materialize/1.0.0/js/materialize.min.js"></script>
    <script>
        // Refresh backup list via AJAX with cache busting
        function refreshBackupList() {
            fetch(window.location.href + "?list=1&rand=" + Math.random(), {
                method: 'GET',
                headers: { 'X-Requested-With': 'XMLHttpRequest' }
            })
            .then(response => response.text())
            .then(html => {
                document.getElementById('backupListContent').innerHTML = html;
            })
            .catch(error => {
                M.toast({html: 'Error refreshing backup list: ' + error, classes: 'red'});
            });
        }
        // CMS Test buttons
        document.querySelectorAll('.testCmsBtn').forEach(function(btn) {
            btn.addEventListener('click', function() {
                var cms = btn.getAttribute('data-cms');
                fetch(window.location.href + "?action=test_cms&cms=" + cms, {
                    method: 'GET',
                    headers: { 'X-Requested-With': 'XMLHttpRequest' }
                })
                .then(response => response.json())
                .then(data => {
                    M.toast({html: cms + " Test: " + data.message, classes: data.message.includes("failed") ? "red" : "green"});
                    document.getElementById('debugLogContent').textContent = data.debug;
                })
                .catch(error => {
                    M.toast({html: "Error: " + error, classes: "red"});
                });
            });
        });
        // Handle CMS backup form via AJAX
        document.getElementById('cmsBackupForm').addEventListener('submit', function(e) {
            e.preventDefault();
            var formData = new FormData(this);
            fetch(window.location.href, {
                method: 'POST',
                body: formData,
                headers: { 'X-Requested-With': 'XMLHttpRequest' }
            })
            .then(response => response.json())
            .then(data => {
                M.toast({html: data.message, classes: data.message.includes("failed") ? "red" : "green"});
                document.getElementById('debugLogContent').textContent = data.debug;
                refreshBackupList();
            })
            .catch(error => {
                M.toast({html: "Error: " + error, classes: "red"});
            });
        });
        // Manual backup test – including password parameter
        document.getElementById('testManualBtn').addEventListener('click', function() {
            var host = document.getElementById('db_host').value;
            var user = document.getElementById('db_user').value;
            var dbname = document.getElementById('db_name').value;
            var pass = document.getElementById('db_password').value;
            fetch(window.location.href + "?manual_test=1&host=" + encodeURIComponent(host) + "&user=" + encodeURIComponent(user) + "&dbname=" + encodeURIComponent(dbname) + "&password=" + encodeURIComponent(pass), {
                method: 'GET',
                headers: { 'X-Requested-With': 'XMLHttpRequest' }
            })
            .then(response => response.json())
            .then(data => {
                M.toast({html: "Manual Test: " + data.message, classes: data.message.includes("failed") ? "red" : "green"});
                document.getElementById('debugLogContent').textContent = data.debug;
            })
            .catch(error => {
                M.toast({html: "Error: " + error, classes: "red"});
            });
        });
        // Handle manual backup form via AJAX
        document.getElementById('manualBackupForm').addEventListener('submit', function(e) {
            e.preventDefault();
            var formData = new FormData(this);
            formData.append('manual_backup', '1');
            fetch(window.location.href, {
                method: 'POST',
                body: formData,
                headers: { 'X-Requested-With': 'XMLHttpRequest' }
            })
            .then(response => response.json())
            .then(data => {
                M.toast({html: data.message, classes: data.message.includes("failed") ? "red" : "green"});
                document.getElementById('debugLogContent').textContent = data.debug;
                refreshBackupList();
            })
            .catch(error => {
                M.toast({html: "Error: " + error, classes: "red"});
            });
        });
        // Delete backup file using event delegation
        document.getElementById('backupListSection').addEventListener('click', function(e) {
            if (e.target && e.target.closest('.deleteBtn')) {
                var btn = e.target.closest('.deleteBtn');
                var fileName = btn.getAttribute('data-file');
                if (confirm("Are you sure you want to delete " + fileName + "?")) {
                    var formData = new FormData();
                    formData.append('delete_zip', fileName);
                    fetch(window.location.href, {
                        method: 'POST',
                        body: formData,
                        headers: { 'X-Requested-With': 'XMLHttpRequest' }
                    })
                    .then(response => response.json())
                    .then(data => {
                        M.toast({html: data.message, classes: data.message.includes("failed") ? "red" : "green"});
                        if (!data.error) { refreshBackupList(); }
                    })
                    .catch(error => {
                        M.toast({html: "Error: " + error, classes: "red"});
                    });
                }
            }
        });
        // Copy URL functionality using event delegation
        document.getElementById('backupListSection').addEventListener('click', function(e) {
            if (e.target && e.target.closest('.copyUrlBtn')) {
                var btn = e.target.closest('.copyUrlBtn');
                var url = btn.getAttribute('data-url');
                // Construct full URL by removing current PHP file from URL and appending backup file name
                var baseUrl = window.location.href.split('?')[0].replace(/\/[^\/]+\.php$/, '');
                var fullUrl = baseUrl + "/" + url;
                navigator.clipboard.writeText(fullUrl)
                .then(() => {
                    M.toast({html: "Download URL copied to clipboard!", classes: "green"});
                })
                .catch(err => {
                    M.toast({html: "Failed to copy URL: " + err, classes: "red"});
                });
            }
        });
    </script>
</body>
</html>
